{-# LANGUAGE BangPatterns #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}

module Data.BenchSumBag (benchmarks) where

import Criterion

import Data.Monoid
import Data.Coerce
import Control.DeepSeq

import qualified Data.SumBag as SumBag
import Data.SumBag (SumBag)

benchmarks :: Benchmark
benchmarks = bgroup "Data.SumBag" [benchInsert, benchDelete, benchEq]

benchInsert :: Benchmark
benchInsert = bgroup "insert" $
  let mkBench n = withBag (range 1 n) $ \(~bag) ->
        bench (show n ++ " elements") $ whnf (SumBag.insert 0) bag
  in map mkBench [1000,2000..10000]
  
benchDelete :: Benchmark
benchDelete = bgroup "delete" $
  let mkBench n = withBag (range 1 n) $ \(~bag) ->
        bench (show n ++ " elements") $ whnf (SumBag.delete 1) bag
  in map mkBench [1000,2000..10000]

benchEq :: Benchmark
benchEq = bgroup "eq" $
  let mkBench n =
        withBag (range 1 n) $ \(~bag) ->
        withBag (reverse $ range 1 n) $ \(~revbag) ->
          bench (show n ++ " elements") $ whnf (uncurry (==)) (bag, revbag)
  in map mkBench [1000,2000..10000]

range :: Int -> Int -> [Sum Int]
range a b = coerce [a..b]

withBag :: [Sum Int] -> (SumBag (Sum Int) -> Benchmark) -> Benchmark
withBag input = env setupEnv
  where setupEnv = do
          let !bag = SumBag.fromList input
          return bag 

instance NFData (SumBag a) where
  rnf a = seq a ()
