{-# LANGUAGE RecordWildCards #-}
{-# LANGUAGE BangPatterns #-}

module Copar.Algorithm.Initialize
  ( initialize
  ) where

import           Control.Monad
import           Control.Monad.ST
import           Data.STRef
import           Data.Ord (comparing)

import qualified Data.Vector as V
import qualified Data.Vector.Mutable as VM

import           Data.MorphismEncoding
import           Data.RefinablePartition (Block)
import           Copar.RefinementInterface      ( RefinementInterface
                                                , Label
                                                , F1
                                                )
import qualified Copar.RefinementInterface as RI
import           Copar.Algorithm.Types
import qualified Data.RefinablePartition as Partition

-- returns (initial queue content, algo state)
initialize :: forall f s. RefinementInterface f
           => Encoding (Label f) (F1 f)
           -> Bool
           -> ST s ([Block], AlgoState s f)
initialize encoding enableOptimization = do
  toSub <- VM.replicate (size encoding) []
  lastW <- VM.new (numEdges encoding)
  predMutable <- VM.replicate (size encoding) []
  f3Cache <- VM.new (size encoding)

  forM_ (edgeRefs encoding) $ \e -> case graph encoding e of
    Edge x _ y -> do
      VM.modify toSub (e:) x
      VM.modify predMutable (e:) y

  forM_ (states encoding) $ \x -> do
    outgoingLabels <-  VM.read toSub x
      >>= mapM (\(!x) -> let !l = label (graph encoding x) in return l)
    px <- newSTRef $! RI.init @f (typeOf encoding x) outgoingLabels

    VM.read toSub x >>= mapM_ (\(EdgeRef e) -> VM.write lastW e px)
    VM.write toSub x []


  -- We can use unsafeFreeze here, since we don't need predMutable anymore
  pred <- V.unsafeFreeze predMutable

  -- Initialize partition with one block and assigning each state to that block
  partition <- Partition.make1 (size encoding)
  -- immediately group according by type
  blocks <- Partition.groupBy partition 0 (comparing (typeOf encoding))

  size1Count <- newSTRef 0
  let size1Enabled = enableOptimization

  return (blocks, AlgoState {..})
