{-# LANGUAGE StrictData #-}
{-# LANGUAGE TemplateHaskell #-}

module Copar.Algorithm.Types
  ( AlgoState(..)
  -- * Lenses
  , toSubL
  , lastWL
  , encodingL
  , predL
  , partitionL
  , f3CacheL
  , size1CountL
  , size1EnabledL
  )
where

import           Data.STRef

import           Data.Vector                    ( Vector )
import           Data.Vector.Mutable            ( MVector )
import           Lens.Micro.TH

import           Data.MorphismEncoding
import           Data.RefinablePartition        ( RefinablePartition )
import           Copar.RefinementInterface

data AlgoState s f = AlgoState
  { toSub :: {-# UNPACK #-} (MVector s [EdgeRef])
  , lastW :: {-# UNPACK #-} (MVector s (STRef s (Weight f)))
  , encoding :: {-# UNPACK #-} (Encoding (Label f) (F1 f))
  , pred :: {-# UNPACK #-} (Vector [EdgeRef])
  , partition :: {-# UNPACK #-} (RefinablePartition s)
  , f3Cache :: {-# UNPACK #-} (MVector s (F3 f))
  -- | How often did we skip a block instead of calling update/splitBlock on it
  -- due to our size-1-optimization.
  , size1Count :: {-# UNPACK #-} (STRef s Int)
  -- | Enable the 1-block optimization
  , size1Enabled :: Bool
  }

makeLensesFor
  [ ( "toSub", "toSubL" )
  , ( "lastW", "lastWL" )
  , ( "encoding", "encodingL" )
  , ( "pred", "predL")
  , ( "partition", "partitionL")
  , ( "f3Cache", "f3CacheL")
  , ( "size1Count", "size1CountL")
  , ( "size1Enabled", "size1EnabledL")
  ]
  ''AlgoState
