{-# LANGUAGE TemplateHaskell #-}
{-# LANGUAGE StrictData #-}

module Copar.Coalgebra.Parser.Internal
  ( ParserState(..)
  , graph
  , f1Map
  , symbolTable
  , nextState
  , disableSanity
  , Symbol(..)
  , initState
  ) where

import qualified Data.HashMap.Strict as M
import           Data.Text (Text)
import           Data.Vector (Vector)
import           Lens.Micro.Platform

import           Data.MorphismEncoding (State, Edge)
import           Copar.FunctorExpression.Sorts (Sorted)

data Symbol = Defined | Undefined
  deriving (Eq)

data ParserState l f1 = ParserState
  { _graph :: [Vector (Edge (Sorted l))]
  , _f1Map :: M.HashMap State (Sorted f1)
  , _symbolTable :: M.HashMap Text (State, Symbol)
  , _nextState :: Int
  , _disableSanity :: Bool -- ^ True if parsers are allowed (encouraged) to
                           -- disable sanity checks on the input
  }
makeLenses ''ParserState

initState :: ParserState l f1
initState = ParserState
  { _graph = []
  , _f1Map = M.empty
  , _symbolTable = M.empty
  , _nextState = 0
  , _disableSanity = False
  }
