{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE AllowAmbiguousTypes #-}

module Copar.FunctorDescription
  ( DynFunctorDescription(..)
  , FunctorParser(..)
  , FunctorDescription(..)
  , formatFunctorDescription
  , formatFunctorDescriptions
  , dynPrecedence
  , dynFunctorDescription
  , dynFunctorExprParser
  )
where

import           Data.Text                      ( Text )
import           Data.Text.Prettyprint          ( Doc
                                                , (<+>)
                                                , AnsiStyle
                                                )
import qualified Data.Text.Prettyprint         as Doc

import           Data.Proxy

import           Copar.FunctorExpression.Parser
import           Copar.Functors.SomeFunctor

class (Suitable f) => FunctorDescription f where
  name :: Text
  syntaxExample :: Text
  description :: Maybe (Doc AnsiStyle)
  precedence :: Int
  functorExprParser :: FunctorParser f

dynFunctorDescription :: forall f. FunctorDescription f => DynFunctorDescription
dynFunctorDescription = DynFunctorDescription (Proxy :: Proxy f)

data DynFunctorDescription where
  DynFunctorDescription :: FunctorDescription f => Proxy f -> DynFunctorDescription

dynPrecedence :: DynFunctorDescription -> Int
dynPrecedence (DynFunctorDescription (Proxy :: Proxy f)) = precedence @f

dynFunctorExprParser :: DynFunctorDescription -> FunctorParser SomeFunctor
dynFunctorExprParser (DynFunctorDescription (Proxy :: Proxy f)) = transParser SomeFunctor (functorExprParser @f)

formatFunctorDescription :: FunctorDescription f => Proxy f -> Doc AnsiStyle
formatFunctorDescription (Proxy :: Proxy f)= Doc.vsep
  [ Doc.annotate (Doc.bold <> Doc.underlined) (Doc.pretty (name @f))
    <+> Doc.parens (Doc.pretty (syntaxExample @f))
  , maybe mempty ((Doc.line <>) . Doc.indent 4) (description @f)
  , ""
  ]

formatFunctorDescriptions :: [DynFunctorDescription] -> Doc AnsiStyle
formatFunctorDescriptions = Doc.vsep . map (\(DynFunctorDescription proxy) -> formatFunctorDescription proxy)
