{-# LANGUAGE AllowAmbiguousTypes #-}
{-# LANGUAGE Strict #-}
{-# LANGUAGE CPP #-}
module Copar.FunctorExpression.Desorting
  ( Desorted
  , Sorted(..)
  , desort
  , mkDesortedLabel
  ) where

import Prelude hiding (init)

import Copar.FunctorExpression.Type
import Copar.FunctorExpression.Sorts
import Copar.RefinementInterface
import Copar.Functors.SomeFunctor

data Desorted f a = Desorted (FunctorExpression f Sort) a
  deriving (Functor, Foldable, Traversable, Show)

desort :: FunctorExpression f Sort -> Desorted f ()
desort expr = Desorted expr ()

type instance F1 (Desorted f) = Sorted (F1 f)
#ifdef RELEASE
type instance Label (Desorted f) = Label f
type instance Weight (Desorted f) = Weight f
type instance F3 (Desorted f) = F3 f
#else
type instance Label (Desorted f) = Sorted (Label f)
type instance Weight (Desorted f) = Sorted (Weight f)
type instance F3 (Desorted f) = Sorted (F3 f)
#endif

mkDesortedLabel :: Sorted (Label f) -> Label (Desorted f)
#ifdef RELEASE
mkDesortedLabel (Sorted _ x) = x
#else
mkDesortedLabel = id
#endif

instance RefinementInterface f => RefinementInterface (Desorted f) where
  {-# SPECIALIZE instance RefinementInterface (Desorted SomeFunctor) #-}

#ifdef RELEASE
  init (Sorted sort f1) labels = init @f f1 labels
#else
  init (Sorted sort f1) labels = Sorted sort (init @f f1 (filterBySort sort labels))
#endif

#ifdef RELEASE
  update labels w =
    let (l, f3, r) = ((update @f) $! labels) $! w
    in (l, f3, r)
#else
  update labels (Sorted sort w) =
    let (l, f3, r) = update @f (filterBySort sort labels) w
    in (Sorted sort l, Sorted sort f3, Sorted sort r)
#endif

#ifndef RELEASE
-- FIXME Don't ignore sort-mismatches. Raise a lound error!
filterBySort :: Sort -> [Sorted x] -> [x]
filterBySort sort = map sortedElem . filter ((==sort) . sortedSort)
#endif
