{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE GeneralizedNewtypeDeriving #-}
{-# LANGUAGE ScopedTypeVariables #-}
module Copar.FunctorExpression.Parser
  ( FunctorParser(..)
  , prefix
  , postfix
  , infixR
  , transParser
  , Precedence(..)
  , functorsParser
  , parseFunctorExpression
  ) where

import           Text.Megaparsec
import           Data.Text (Text)

import           Copar.FunctorExpression.Type
import qualified Copar.Parser.Lexer as L
import           Copar.Parser.Types

data FunctorParser f where
  FunctorParser :: (forall a m. MonadParser m => m a -> m (Either a (f a))) -> FunctorParser f

prefix :: (forall a m. MonadParser m => m (a -> f a)) -> FunctorParser f
prefix parser = FunctorParser $ \inner -> do
  (parser >>= \f -> Right . f <$> inner) <|> (Left <$> inner)

postfix :: (forall a m. MonadParser m => m (a -> f a)) -> FunctorParser f
postfix parser = FunctorParser $ \inner -> do
  i <- inner
  (parser >>= \f -> return (Right (f i))) <|> return (Left i)

infixR :: (forall a m. MonadParser m => m (a -> a -> f a)) -> FunctorParser f
infixR parser = FunctorParser $ \inner -> do
  l <- inner
  let p = do
        f <- parser
        r <- inner
        return (Right (f l r))

  p <|> return (Left l)

transParser :: (forall a. f a -> g a) -> FunctorParser f -> FunctorParser g
transParser natTrans (FunctorParser p) =
  -- yeah! Tripple fmap
  FunctorParser (fmap (fmap (fmap natTrans)) p)

newtype Precedence = Precedence Int
  deriving (Num, Bounded, Enum, Integral, Real, Ord, Eq, Show)

functorsParser :: forall f m. [FunctorParser f] -> ParserT m (FunctorExpression f Precedence)
functorsParser functors = parseLevel (zip (reverse functors) [1..])
  where
    parseLevel :: [(FunctorParser f, Precedence)] -> ParserT m (FunctorExpression f Precedence)
    parseLevel ((FunctorParser f, i):functors) = f (parseLevel functors) >>= \case
      Left a -> return a
      Right fa -> return (Functor i fa)
    parseLevel [] = variable <|> L.parens (functorsParser functors)

    variable :: ParserT m (FunctorExpression f Precedence)
    variable = L.symbol "X" >> return Variable
                 <?> "variable (X)"

parseFunctorExpression ::
     [FunctorParser f]
  -> String
  -> Text
  -> Either ParseErr (FunctorExpression f Precedence)
parseFunctorExpression functors =
  parse ((L.space *> L.newlines) *> functorsParser functors <* eof)
