module Copar.FunctorExpression.Printable 
( Printable (..)
, PrintConfig (..)
, printTree
, defaultPrintConfig
) where

import           Data.Text.Lazy.Builder as Build

-- | Print the required skeleton for the current scope.
--
-- The current scope has already been popped of the stack.
printSkeleton :: [a] -> [[a]] -> Build.Builder
printSkeleton h xs =
  foldl (\str e -> (if null e then "  " else "│ ") <> str)
    (if null h then "└ " else "├ ")
    xs

printTree :: (a -> ([(Build.Builder, a)], Build.Builder)) -> a -> Build.Builder
printTree f head =
  let (children, functor) = f head
  in functor <> printInner f [children]

printInner :: (a -> ([(Build.Builder, a)], Build.Builder)) -> [[(Build.Builder, a)]] -> Build.Builder
printInner f (((scope, v):h):rs) =
    "\n" <> printSkeleton h rs <> scope <> ": "
    <> functor <> (printInner f (children:h:rs))
  where
    (children, functor) = f v
printInner f ([]:rs) = printInner f rs
printInner _ [] = mempty

data PrintConfig = PrintConfig { withName :: Build.Builder -> Build.Builder }

defaultPrintConfig :: PrintConfig
defaultPrintConfig = PrintConfig {
  withName = \name -> name <> " "
}

class Printable f where
  -- | The operator precedence of the given functor,
  --
  -- This is used to elide some unnecessary braces and should
  -- be kept up to date with `registeredFunctors`.
  precedence :: f a -> Int

  printMe :: PrintConfig -> f Build.Builder -> Build.Builder
