module Copar.FunctorPrinter (printFunctor, printParseableFunctor, Printable) where

import           Data.Text as T
import           Data.Text.Lazy as Lazy
import           Data.Text.Lazy.Builder as Build

import           Copar.FunctorExpression.Printable
import           Copar.FunctorExpression.Sorts (Sort, formatSortAsScope)
import           Copar.FunctorExpression.Type (FunctorExpression (..))

printFunctor :: (Printable f, Foldable f, Functor f) => FunctorExpression f Sort -> T.Text
printFunctor (Functor _ f') = Lazy.toStrict (Build.toLazyText (printTree inner f'))
printFunctor Variable = "Variable X"

inner :: (Printable f, Foldable f, Functor f)
    => f (FunctorExpression f Sort)
    -- ([('sort, subfunctor)], functor)
    -> ([(Build.Builder, f (FunctorExpression f Sort))], Build.Builder)
inner f = (foldMap getChild f, print (fmap printChild f))
  where
    print = printMe PrintConfig { withName = \name -> name <> " " }

    printChild Variable = "X"
    printChild (Functor sort _) = formatSortAsScope sort

    getChild Variable = []
    getChild (Functor sort f') = [(formatSortAsScope sort, f')]

printParseableFunctor :: (Printable f, Foldable f, Functor f) => FunctorExpression f Sort -> T.Text
printParseableFunctor (Functor _ f') = Lazy.toStrict (Build.toLazyText (print (fmap (printInner (precedence f')) f')))
  where
    print :: forall f. (Printable f) => f Build.Builder -> Build.Builder
    print = printMe PrintConfig { withName = \_ -> "" }

    printInner :: (Printable f, Foldable f, Functor f) => Int -> FunctorExpression f Sort -> Build.Builder
    printInner _ Variable = "X"
    printInner prec (Functor _ inner) = 
      let inner_prec = precedence inner
      in if inner_prec > prec 
        then print (fmap (printInner inner_prec) inner)
        else "(" <> print (fmap (printInner inner_prec) inner) <> ")"

printParseableFunctor Variable = "X"

