module Copar.Timing (withTime, showTimeDiff) where

import           System.CPUTime

import           Data.Text                      ( Text )
import qualified Data.Text                     as T
import           Data.Scientific

-- | Run an IO action and return its result together with its execution time in
-- picoseconds.
withTime :: IO a -> IO (Integer, a)
withTime action = do
  time1 <- getCPUTime
  res   <- action
  time2 <- getCPUTime
  return (time2 - time1, res)

-- | Return the time as seconds
showTimeDiff :: Integer -> Text
showTimeDiff picoseconds =
  T.pack (formatScientific Fixed (Just 10) (scientific picoseconds (-12)))
