{-# LANGUAGE BangPatterns #-}
{-# LANGUAGE ScopedTypeVariables #-}

module Data.Algorithm.PossibleMajorityCandidate where

import Data.Foldable

import qualified Data.Vector.Unboxed as VU

-- | Compute the possible majority candidate in t
--
-- The result is undefined if t is empty
--
-- Runtime is O(n)
possibleMajorityCandidate :: forall a t. (Eq a, Foldable t) => t a -> a
possibleMajorityCandidate = snd . foldl' loop (0, undefined)
  where
    loop :: (Int, a) -> a -> (Int, a)
    loop (!count,pmc) !a
      | count == 0  = (1, a)
      | a == pmc    = (count+1, pmc)
      | otherwise   = (count-1, pmc)
{-# INLINE possibleMajorityCandidate #-}

possibleMajorityCandidateBy' :: forall a b. (VU.Unbox a, Eq b) => (a -> b) -> VU.Vector a -> b
possibleMajorityCandidateBy' f = snd . VU.foldl' loop (0, undefined)
  where
    loop :: (Int, b) -> a -> (Int, b)
    loop (!count,pmc) !a
      | count == 0  = (1, f a)
      | f a == pmc    = (count+1, pmc)
      | otherwise   = (count-1, pmc)
{-# INLINEABLE possibleMajorityCandidateBy' #-}
