{-# LANGUAGE GeneralizedNewtypeDeriving #-}

-- | Newtype wrappers around 'Bits' implementors to provide monoid instances
-- with '.&.' and '.|.'.
module Data.Bits.Monoid (BitAnd(..), BitOr(..)) where

import           Data.Bits
import           Data.Semigroup                 ( Semigroup(..)
                                                , stimesMonoid
                                                )
import           Control.DeepSeq

-- | Wrapper that provides a 'Semigroup'/'Monoid' instances for the wrapped type
-- using its 'Bits' instance:
--
-- * 'mempty' is all bits set
-- * '<>' is '.&.'
newtype BitAnd a = BitAnd { getBitAnd :: a }
  deriving (Eq, Ord, Show, Read, Num, Real, Enum, Integral, Bits, NFData)

instance (Bits a) => Semigroup (BitAnd a) where
  (<>)   = (.&.)
  stimes = stimesMonoid

instance (Bits a) => Monoid (BitAnd a) where
  mempty = (complement zeroBits)


-- | Wrapper that provides a 'Semigroup'/'Monoid' instances for the wrapped type
-- using its 'Bits' instance:
--
-- * 'mempty' is all bits cleared
-- * '<>' is '.|.'
newtype BitOr a = BitOr { getBitOr :: a }
  deriving (Eq, Ord, Show, Read, Num, Real, Enum, Integral, Bits, NFData)

instance (Bits a) => Semigroup (BitOr a) where
  (<>)   = (.|.)
  stimes = stimesMonoid

instance (Bits a) => Monoid (BitOr a) where
  mempty = zeroBits
