{-# LANGUAGE TypeApplications #-}
{-# LANGUAGE OverloadedStrings #-}

module Hardwired where

import Stats

import Data.Text(Text)
import qualified Data.Text as T

import           Copar.Algorithm
import System.IO
import System.Exit

import           Data.Proxy
import           Control.Exception
import qualified Copar.Parser as P
import qualified Data.MorphismEncoding as Encoding
import           Copar.Functors.GroupValued (GroupValued(GroupValued))
import           Copar.Functors.Powerset (Powerset(Powerset))
import           Text.Megaparsec (parse)
import           Text.Megaparsec.Error (errorBundlePretty)
import           Copar.FunctorExpression.Sorts
import           Copar.FunctorExpression.Desorting (Desorted(..))
import           Copar.FunctorExpression.Type
import           Copar.RefinementInterface (RefinementInterface, Label, F1)
import           Copar.Timing
import           Copar.Coalgebra.Parser (morphismsParser)
import qualified Data.Text.IO                  as TextIO
import           Data.Bifunctor
import           Data.Float.Utils (EqDouble)
import           Data.Partition (Partition)

{-# SPECIALIZE refineWithStats :: Proxy (Desorted Powerset) -> Encoding.Encoding (Label (Desorted Powerset)) (F1 (Desorted Powerset)) -> Bool -> IO (Partition, AlgoStatistics) #-}


main stats inputFile partitionPrinter enableOptimizations = do
      withTimeStat stats "overall-duration" $ do
        let myfunctor = (annotateSorts $ Functor () (Powerset Variable))
        -- let mp = morphismsParser @(GroupValued EqDouble)
        let mp = morphismsParser @(Powerset)
        let morphParser = mp myfunctor P.DisableSanityChecks
        let f = myfunctor
        (symbolTable, encoding) <- withTimeStat stats "parse-duration" $ do
          let (filename,getFilecontents) = case inputFile of
                                           Nothing -> ("(stdin)", TextIO.getContents)
                                           Just "-" -> ("(stdin)", TextIO.getContents)
                                           Just f -> (f, TextIO.readFile f)
          contents <- getFilecontents
          case parse morphParser filename contents of
              Left  err -> hPutStrLn stderr (errorBundlePretty err) >> exitFailure
              Right res -> evaluate res

        logStat stats "states" (tshow (Encoding.size encoding))
        logStat stats "edges"  (tshow (Encoding.numEdges encoding))

        (part, algoStats) <- withTimeStat stats "algorithm-duration" $
                  refineWithStats @(Desorted Powerset) Proxy encoding enableOptimizations
        logStat stats
                "initial-partition-size"
                (tshow (initialBlocks algoStats))
        logStat stats
                "initialize-duration"
                (showTimeDiff (initTime algoStats))
        logStat stats "split-operation-count" (tshow (splitCount algoStats))
        logStat stats
                "refine-duration"
                (showTimeDiff (refineTime algoStats))
        logStat stats
                "size1-skipped"
                (tshow (size1Skipped algoStats))

        withTimeStat
          stats
          "output-duration"
          (partitionPrinter encoding symbolTable part)
        return ()

      finalizeStats stats

tshow :: Show a => a -> Text
tshow = T.pack . show
