{-# LANGUAGE OverloadedStrings #-}
module Stats where

import           Copar.Timing

import           System.IO
import           System.Exit
import           Control.Arrow ((***))
import           Text.JSON
import           Data.IORef
import           Data.Map (Map)
import qualified Data.Map as M
import           Data.Text (Text)
import qualified Data.Text as T
import qualified Data.Text.IO as T

----------------------------------------------------------------------
-- Statistics logging
----------------------------------------------------------------------

data StatType = NoStats | TextStats | JsonStats

data Stats = Stats
  { statsType :: StatType
  , statsState :: IORef (Map Text Text)
  }

initStats :: Bool -> Bool -> IO Stats
initStats statsText statsJson =
  case (statsText, statsJson) of
    (False, False) -> return (Stats NoStats undefined)
    (True, False) -> return (Stats TextStats undefined)
    (False, True) -> Stats JsonStats <$> newIORef M.empty
    (True, True) -> do
      hPutStrLn
        stderr
        "Invalid command line options: --stats and --stats-json cannot be used together."
      exitFailure

logStat :: Stats -> Text -> Text -> IO ()
logStat stats key value =
  case statsType stats of
    NoStats -> return ()
    TextStats -> T.hPutStrLn stderr $ key <> ": " <> value
    JsonStats -> modifyIORef (statsState stats) (M.insert key value)

finalizeStats :: Stats -> IO ()
finalizeStats stats =
  case statsType stats of
    JsonStats -> do
      m <- readIORef (statsState stats)
      let json =
            toJSObject
              (map
                 (T.unpack *** (JSString . toJSString . T.unpack))
                 (M.toList m))
      hPutStrLn stderr $ showJSObject json ""
    _ -> return ()

withTimeStat :: Stats -> Text -> IO a -> IO a
withTimeStat stats name action = case statsType stats of
  NoStats -> action
  _ -> do
    (duration, res) <- withTime action
    logStat stats name (showTimeDiff duration)
    return res

