{-# LANGUAGE TypeApplications #-}

module Parser
  ( Parser
  , lexeme
  , decimalP
  , doubleP
  , rationalP
  , nameP
  , symbolP
  ) where

import           Data.Char
import           Data.Void (Void)

import           Text.Megaparsec
import qualified Text.Megaparsec.Char.Lexer as L
import           Text.Megaparsec.Char
import           Data.Text (Text)
import qualified Data.Text as T

type Parser = Parsec Void Text

lexeme :: Parser a -> Parser a
lexeme = L.lexeme space

decimalP :: Parser Int
decimalP = lexeme L.decimal

doubleP :: Parser Double
doubleP = lexeme (try L.float <|> (fromIntegral @Int <$> L.decimal))

-- | Parse a decimal into a ratio
rationalP :: Parser Rational
rationalP = toRational <$> lexeme L.scientific

isName1 :: Char -> Bool
isName1 c = isLetter c || c == '_'

isName :: Char -> Bool
isName c = isAlphaNum c || c == '_'

nameP :: Parser Text
nameP = lexeme $
  T.cons <$> satisfy isName1 <*> takeWhileP (Just "name") isName

symbolP :: Text -> Parser Text
symbolP = L.symbol space
