module DotOutput (dotB) where

import           Data.Foldable                  ( fold )
import           Data.Word

import qualified Data.Text.Lazy.Builder        as Build
import qualified Data.Text.Lazy.Builder.Int    as Build
import           Lens.Micro.Platform

import           Type

dotB :: DFA -> Build.Builder
dotB dfa = "digraph {\n" <> nodesB dfa <> transitionsB dfa <> "}\n"


nodesB :: DFA -> Build.Builder
nodesB dfa = foldMap (nodeB dfa) [0 .. dfa ^. states - 1]


nodeB :: DFA -> Word32 -> Build.Builder
nodeB dfa n = "  \"n" <> Build.decimal n <> "\" [shape = " <> shape <> "];\n"
  where
    shape =
      if dfa ^?! isFinal . ix (fromIntegral n) then "doublecircle" else "circle"


transitionsB :: DFA -> Build.Builder
transitionsB dfa = foldMap (transitionsForB dfa) [0 .. dfa ^. states - 1]


transitionsForB :: DFA -> Word32 -> Build.Builder
transitionsForB dfa n = fold
  (zipWith (transitionB n)
           [0 ..]
           (dfa ^.. transitions . ix (fromIntegral n) . each))


transitionB :: Word32 -> Word32 -> Word32 -> Build.Builder
transitionB source letter target =
  "  n"
    <> Build.decimal source
    <> " -> n"
    <> Build.decimal target
    <> " [label=a"
    <> Build.decimal letter
    <> "];\n"

