{-# LANGUAGE TemplateHaskell #-}

module Type
  ( DFA(..)
  , HasStates(..)
  , HasLetters(..)
  , HasTransitions(..)
  , HasIsFinal(..)
  ) where

import           Data.Word

import           Data.Vector                    ( Vector )
import qualified Data.Vector.Unboxed           as VU
import           Lens.Micro.TH

-- The objective here is to support large DFAs and thus is might seem backwards
-- to restrict the number of states and letters to Word32. In reality, the
-- limiting factor is RAM, e.g 10000x10000 requires already more ram than my
-- 16GB machine has. Word32 requires half as much RAM than Int, so it allows for
-- larger graphs in practice.

data DFA = DFA
  { dfaStates :: Word32
  , dfaLetters :: Word32
  , dfaTransitions :: Vector (VU.Vector Word32)
  , dfaIsFinal :: VU.Vector Bool
  }

makeLensesWith abbreviatedFields ''DFA
