module ValmariOutput (valmariB) where

import           Data.Monoid
import           Data.Word

import qualified Data.Text.Lazy.Builder        as Build
import qualified Data.Text.Lazy.Builder.Int    as Build
import           Lens.Micro.Platform
import qualified Data.Vector.Unboxed           as VU
import qualified Data.Vector                   as V

import           Type

valmariB :: DFA -> Build.Builder
valmariB dfa = sizesB dfa <> transitionsB dfa <> blocksB dfa


sizesB :: DFA -> Build.Builder
sizesB dfa =
  Build.decimal (dfa ^. states)
    <> " "
    <> Build.decimal (dfa ^. letters)
    <> " "
    <> Build.decimal ((dfa ^. letters) * (dfa ^. states))
    <> " 0 " -- "2\n" -- always 2 blocks
    <> (if VU.and (dfa ^. isFinal) || not (VU.or (dfa ^. isFinal))
         then "1"
         else "2"
       )
    <> "\n"


transitionsB :: DFA -> Build.Builder
transitionsB dfa = foldMap
  (uncurry (transB dfa))
  [ (from, letter)
  | from   <- [0 .. dfa ^. states - 1]
  , letter <- [0 .. dfa ^. letters - 1]
  ]


transB :: DFA -> Word32 -> Word32 -> Build.Builder
transB dfa from letter =
  Build.decimal (from + 1)
    <> " "
    <> Build.decimal (letter + 1)
    <> " "
    <> Build.decimal (to + 1)
    <> "\n"
  where
    to =
      dfa ^?! transitions . ix (fromIntegral from) . ix (fromIntegral letter)


blocksB :: DFA -> Build.Builder
blocksB dfa | VU.and (dfa ^. isFinal) = statesThatAre final
            | otherwise = statesThatAre (not . final)
  where
    statesThatAre p =
      foldMap stateB (filter p [0 .. dfa ^. states - 1]) <> "0 \n"
    stateB x = Build.decimal (x + 1) <> " "
    final x = dfa ^?! isFinal . ix (fromIntegral x)
