{-# OPTIONS_GHC -fno-warn-orphans  #-}

module Main (main) where

import           Test.Hspec
import           Test.QuickCheck

import qualified Data.Vector                   as V
import           Data.List                      ( nub )

import           Types
import qualified IndexedTransition
import           Generator

main :: IO ()
main = hspec $ do
  describe "IndexedTransition" $ do
    it "is isomorphic to a transition (10 states, signature 4xX)" $ do
      let wta = WTASpec MaxInt 10 (V.fromList [0, 4])
      property $ \t ->
        uncurry (IndexedTransition.index wta)
                (IndexedTransition.fromIndex wta t)
          == t

    it "is isomorphic to a transition (50 states, signature 4xX^2)" $ do
      let wta = WTASpec MaxInt 50 (V.fromList [0, 0, 4])
      property $ \t ->
        uncurry (IndexedTransition.index wta)
                (IndexedTransition.fromIndex wta t)
          == t

    describe "maxIndex" $ do
      it "doesn't overflow for large values" $ do
        let wta = WTASpec MaxInt 72000 (V.fromList [0, 0, 0, 0, 4])
        IndexedTransition.maxIndex wta `shouldSatisfy` (> 0)

  describe "Generator" $ describe "uniqueTransitions" $ do
    it "generates unique transitions" $ do
      res <- uniqueTransitions 5 (IndexedTransition.Index 10)
      nub res `shouldBe` res

      res <- uniqueTransitions 100 (IndexedTransition.Index 120)
      nub res `shouldBe` res

    it "generates the correct number of transitions" $ do
      (length <$> uniqueTransitions 5 (IndexedTransition.Index 10))
        `shouldReturn` 5

      (length <$> uniqueTransitions 20 (IndexedTransition.Index 21))
        `shouldReturn` 20

      (length <$> uniqueTransitions 100 (IndexedTransition.Index 101))
        `shouldReturn` 100

    it "doesn't generate a transition greater than max" $ do
      res <- maximum <$> uniqueTransitions 100 (IndexedTransition.Index 150)
      res `shouldSatisfy` (< 150)


instance Arbitrary IndexedTransition.IndexedTransition where
  arbitrary = IndexedTransition.Index <$> arbitrary
