{-# LANGUAGE GADTs #-}
{-# LANGUAGE OverloadedStrings #-}

-- | Generator for the input format of the implementation of WTA bisimulation at
-- https://people.cs.umu.se/~johanna/bisimulation/
--
-- Note that to the best of my knowledge, only supports the powerset functor
-- with weights that are textually hardcoded as 1.0.
module Hoegberg (hoegbergWTA) where

import qualified Data.Vector                   as V
import           Data.Vector                    ( Vector )
import           Data.Text.Lazy.Builder         ( Builder )
import qualified Data.Text.Lazy.Builder.Int    as Build
import           Data.Foldable
import           Data.List

import           Types

declareState :: WTASpec Bool -> Builder
declareState wta =
  "# states\n"
    <> (fold $ intersperse " " $ map Build.decimal [0 .. numStates wta - 1])

rules :: WTA Bool -> Builder
rules wta =
  "# rules\n" <> (fold $ V.imap transitionsForState (stateTransitions wta))

transitionsForState :: Int -> Vector (Transition Bool) -> Builder
transitionsForState from = foldMap (transition from)

transition :: Int -> Transition Bool -> Builder
transition from trans =
  "sym"
    <> Build.decimal (symbol trans)
    <> " "
    <> Build.decimal from
    <> " 1.0 "
    <> (fold $ intersperse " " $ map Build.decimal (toList (successors trans)))
    <> "\n"

accepting :: WTA Bool -> Builder
accepting wta =
  "# accepting states\n"
    <> (foldMap accepting1 $ V.elemIndices True (stateValue wta))

accepting1 :: Int -> Builder
accepting1 s = Build.decimal s <> " 1.0\n"

-- | Print a WTA. See the module level documentation for why this only accepts
-- "Bool" as monoid.
hoegbergWTA :: WTA Bool -> Builder
hoegbergWTA wta =
  declareState (spec wta) <> "\n" <> rules wta <> "%%%\n" <> accepting wta
