module Copar.FunctorPrinterSpec (spec) where

import Test.Hspec

import Data.Bifunctor
import Data.Either
import Data.Text (unpack, Text)

import Copar.Functors.SomeFunctor
import Copar.FunctorExpression.Type
import Copar.FunctorExpression.Sorts
import Copar.FunctorPrinter
import Copar.Parser

spec :: Spec
spec = do
  printParseableFunctorSpec

printParseableFunctorSpec:: Spec
printParseableFunctorSpec = describe "printParseableFunctor" $ do
  let
    idFunctor :: FunctorExpression SomeFunctor Sort -> Either String (FunctorExpression SomeFunctor Sort)
    idFunctor = parseFunctor "" . printParseableFunctor

    it_is_id :: Text -> Spec
    it_is_id s = it (unpack ("is the identity for `" <> s <> "`")) $
      let res = parseFunctor "" s
      in do
        res `shouldSatisfy` isRight
        res `shouldBe` (res >>= idFunctor)

    it_is_eq :: Text -> Text -> Spec
    it_is_eq s expected = it (unpack ("returns `" <> expected <> "` for `" <> s <> "`")) $
      let res = printParseableFunctor <$> (parseFunctor "" s)
      in do
        res `shouldSatisfy` isRight
        res `shouldSatisfy` \res -> fromRight False ((== expected) <$> res)

  it_is_id "D(PX)"
  it_is_id "{n, f}xPX"
  it_is_id "D({f,n}xPX)"
  it_is_id "D({f,n}xP(DX)^{a, b, c}+({f, n}xX + (Z, min)^({f ,n})))"
  it_is_id "(Z, min)^X"
  it_is_id "X^2"
  it_is_id "P(PX)"
  it_is_id "PX^2"
  it_is_id "P(X^2)"

  it_is_eq "DPX" "DPX"
  it_is_eq "D({f,n}xP(DX)^{a, b, c}+({f, n}xX + (Z, min)^({f ,n})))" "D({f, n}xP(DX)^{a, b, c} + ({f, n}xX + (Z, min)^({f, n})))"
  it_is_eq "(Z, min)^((R, min)^(Z, min)^({f, n} x DX))" "(Z, min)^((R, min)^(Z, min)^({f, n}xDX))"
  it_is_eq "P(PX)" "P(PX)"

  it "correctly prints a variable as X" $
    printParseableFunctor @SomeFunctor Variable `shouldBe` "X"
