module Copar.Functors.BagSpec (spec) where

import           Test.Hspec
import           Test.Hspec.Megaparsec
import           TestHelpers

import           Data.Proxy
import           Control.Monad.ST

import qualified Data.Partition as Part
import           Copar.Algorithm
import           Copar.Coalgebra.Parser
import           Copar.FunctorDescription
import           Copar.FunctorExpression.Desorting
import           Copar.FunctorExpression.Parser
import           Copar.FunctorExpression.Type
import           Copar.Functors.Bag

spec :: Spec
spec = do
  functorExpressionSpec
  parseMorphismPointSpec
  refineSpec

functorExpressionSpec :: Spec
functorExpressionSpec = describe "functorExpression" $ do
  it "parses a normal B correctly" $
    parseFunctorExpression [functorExprParser @Bag] "" "BX" `shouldParse` (Functor 1 (Bag Variable))


  it "parses a unicode B (Ɓ) correctly" $
    parseFunctorExpression [functorExprParser @Bag] "" "ƁX" `shouldParse` (Functor 1 (Bag Variable))

parseMorphismPointSpec :: Spec
parseMorphismPointSpec = describe "parseMorphismPoint" $ do
  it "works for a simple example"
    $ (snd <$> parseMorphisms (Functor 1 (Bag Variable))
                              EnableSanityChecks
                              ""
                              "x: {x,x}"
      )
    `shouldParse` (encoding [(Sorted 1 2)]
                            [(0, (Sorted 1 1), 0), (0, (Sorted 1 1), 0)]
                  )

refineSpec :: Spec
refineSpec = describe "refining" $ do
  let f = Functor 1 (Bag Variable)

  it "distinguishes points with different successor count"
    $ let Right (_, enc) =
            parseMorphisms f EnableSanityChecks "" "x: {x, x, y}\ny: {x, y}"
      in  stToIO (refine (Proxy @(Desorted Bag)) enc True)
            `shouldReturn` (Part.fromBlocks [[0], [1]])
