module Data.Algorithm.PossibleMajorityCandidateSpec (spec) where

import Test.Hspec
import Test.QuickCheck

import Data.Maybe
import Data.List

import Data.Algorithm.PossibleMajorityCandidate

spec :: Spec
spec = do
  it "returns the element that has more then length/2 occurences" $
    -- limit to 30 successful tests, because most of the randomly generated
    -- lists don't have the property that one element occurs more than length/2
    -- times and quickcheck gives up trying to make enough tests positive.
    property positive

  it "returns an element from the list if no element has more than length/2 occurences" $
    property negative

positive :: PMCList Int -> Property
positive vals' =
  let
    vals = toList vals'
    pmc = naivePMC vals
  in pmc /= Nothing ==> possibleMajorityCandidate vals === fromJust pmc

negative :: NonEmptyList Int -> Property
negative (NonEmpty vals) =
  let pmc = naivePMC vals
  in pmc == Nothing ==> possibleMajorityCandidate vals `elem` vals


naivePMC :: [Int] -> Maybe Int
naivePMC vals =
  let
    halfLen = length vals `div` 2
    occurences = group (sort vals)
    pmc = head <$> find (\e -> length e > halfLen) occurences
  in
    pmc

newtype PMCList a = PMCList { toList :: [a] }
  deriving (Show)

instance Arbitrary a => Arbitrary (PMCList a) where
  arbitrary = do
    list <- arbitrary
    pmc <- arbitrary
    PMCList <$> shuffle (list ++ replicate (length list) pmc)
