{-# LANGUAGE GeneralizedNewtypeDeriving #-}

{-# OPTIONS_GHC -fno-warn-orphans #-}

module Data.Bits.MonoidSpec (spec) where

import Test.Hspec
import Test.QuickCheck
import Test.QuickCheck.Classes

import Data.Foldable
import Data.Proxy

import Data.Bits.Monoid

spec :: Spec
spec = do
  andSpec
  orSpec

deriving instance (Arbitrary a) => Arbitrary (BitAnd a)
deriving instance (Arbitrary a) => Arbitrary (BitOr a)

andSpec :: Spec
andSpec = describe "and" $ do
  it "works for a simple example" $
    let res = 0xCAACBACE :: BitAnd Word
    in 0xdeadbeef <> 0xcafeface `shouldBe` res

  lawsToSpec (semigroupLaws (Proxy @(BitAnd Word)))
  lawsToSpec (commutativeSemigroupLaws (Proxy @(BitAnd Word)))
  lawsToSpec (monoidLaws (Proxy @(BitAnd Word)))
  lawsToSpec (commutativeMonoidLaws (Proxy @(BitAnd Word)))

orSpec :: Spec
orSpec = describe "or" $ do
  it "works for a simple example" $
    let res = 0xDEFFFEEF :: BitOr Word
    in 0xdeadbeef <> 0xcafeface `shouldBe` res

  lawsToSpec (semigroupLaws (Proxy @(BitOr Word)))
  lawsToSpec (commutativeSemigroupLaws (Proxy @(BitOr Word)))
  lawsToSpec (monoidLaws (Proxy @(BitOr Word)))
  lawsToSpec (commutativeMonoidLaws (Proxy @(BitOr Word)))

lawsToSpec :: Laws -> Spec
lawsToSpec laws = context ("laws for " ++ lawsTypeclass laws) $
  traverse_ lawToSpec (lawsProperties laws)

  where lawToSpec (name, prop) = specify name (property prop)
