{-# OPTIONS_GHC -fno-warn-orphans #-}

module Data.BlockQueueSpec (spec) where

import           Prelude hiding (null, elem)

import           Test.Hspec
import           Test.QuickCheck

import           Control.Monad.ST
import qualified Data.List as List (delete)

import           Data.BlockQueue
import           Data.RefinablePartition (Block(..))

spec :: Spec
spec = do
  emptySpec
  enqueueSpec

emptySpec :: Spec
emptySpec = describe "empty" $ do
  it "creates an empty queue" $
    property $ \size -> size >= 0 ==> runST $ empty size >>= null

  it "creates a queue where no block is present" $
    property $ \size -> size >= 0 ==> runST $ do
      q <- empty size
      all not <$> mapM (\b -> Block b `elem` q) [0..size-1]

enqueueSpec :: Spec
enqueueSpec = describe "enqueue" $ do
  it "makes an empty queue non-empty"
    $          runST (empty 2 >>= \q -> enqueue q 0 >> null q)
    `shouldBe` False

  it "is inverse to enqueue"
    $ let enqdeq a b = do
            q <- empty 10
            enqueue q a
            enqueue q b
            (,) <$> dequeue q <*> dequeue q
      in  property $ forAll (elements [0 .. 9]) $ \a ->
            forAll (elements (List.delete a [0 .. 9]))
              $ \b -> runST (enqdeq a b) === (Just a, Just b)

  it "doesn't insert blocks twice"
    $ let enqtwice = do
            q <- empty 5
            enqueue q 0
            enqueue q 0
            Just 0 <- dequeue q
            null q
      in  runST enqtwice `shouldBe` True
