{-# OPTIONS_GHC -Wno-orphans #-}

module Data.Vector.Unboxed.Mutable.UtilsSpec (spec) where

import           Test.Hspec
import           Test.QuickCheck

import           Control.Monad.ST
import           Data.List (sort)
import qualified Data.List as L
import           Control.Arrow ((***))
import           Data.Ord (comparing)

import qualified Data.Vector.Unboxed as V

import           Data.Vector.Unboxed.Mutable.Utils

spec :: Spec
spec = do
  partitionSpec
  groupBySpec

partitionSpec :: Spec
partitionSpec = describe "partition" $
  it "does the same as partition on immutable vectors" $
    property $ \(vec :: V.Vector Int) ->
      sortBoth (partition' (<10) vec) === sortBoth (V.partition (<10) vec)

sortBoth :: (Ord a, V.Unbox a) => (V.Vector a, V.Vector a) -> ([a], [a])
sortBoth = sort . V.toList *** sort . V.toList

-- non-ST version of partition
partition' :: V.Unbox a => (a -> Bool) -> V.Vector a -> (V.Vector a, V.Vector a)
partition' predicate vec = runST $ do
  vec' <- V.thaw vec
  s <- partition vec' predicate 0 (V.length vec)
  V.splitAt s <$> V.freeze vec'

groupBySpec :: Spec
groupBySpec = describe "groupBy" $
  it "does the same as Data.List.groupBy" $
    property $ \(lst :: [Int]) ->
      let accuLength = tail' . init . scanl (\x l -> x+length l) 0
      in
        groupBy' (V.fromList lst) id === accuLength (L.group lst)

-- version of tail that can handle empty lists
tail' :: [a] -> [a]
tail' [] = []
tail' (_:tl) = tl

groupBy' :: (Ord b, V.Unbox a) => V.Vector a -> (a -> b) -> [Int]
groupBy' vec predicate = runST $ do
  vec' <- V.thaw vec
  groupBy vec' (comparing predicate) 0 (V.length vec)

instance (Arbitrary a, V.Unbox a) => Arbitrary (V.Vector a) where
  arbitrary = V.fromList <$> arbitrary
