{-# LANGUAGE LambdaCase #-}

module Main (main) where

import           Control.Monad.ST
import           Data.List

import           Data.List.Extra (dropSuffix)
import qualified Data.Text as T
import           System.Directory
import           Test.Hspec

import           Copar.Algorithm
import qualified Copar.Parser as P
import           Copar.PartitionPrinter

main :: IO ()
main = do
  let exampleDir = "examples/"
  files <- filter isOutput <$> listDirectory exampleDir
  hspec $ testFiles (map (exampleDir ++) files)

outSuffix :: String
outSuffix = ".out"

isOutput :: FilePath -> Bool
isOutput = (outSuffix `isSuffixOf`)

testFiles :: [FilePath] -> Spec
testFiles = mapM_ testFile

testFile :: FilePath -> Spec
testFile outfile =
  let file = dropSuffix outSuffix outfile
   in specify file $ do
        output <- readFile (file ++ outSuffix)
        process file `shouldReturn` output

-- | FIXME: Run each example twice, once with and once without transformations.
process :: FilePath -> IO String
process file =
  P.readFile P.defaultConfig file
    >>= \case
          Left  err -> return err
          Right (f, (symTab, enc)) -> do
            partition <- stToIO (refine f enc True)
            return $ T.unpack (showPartition enc symTab partition)
