{-# LANGUAGE FlexibleContexts #-}
module Copar.Parser.BenchLexer (benchmarks) where


import           Criterion

import           Text.Megaparsec
import           Data.Text (Text)
import           Control.DeepSeq (NFData)

import           Copar.Parser.Types
import qualified Copar.Parser.Lexer as L

benchmarks :: Benchmark
benchmarks = bgroup "Copar.Parser.Lexer"
  [ benchColon
  , benchDecimal
  , benchFloat
  , benchHex
  , benchName
  , benchWhitespace
  ]

benchColon :: Benchmark
benchColon = bgroup "colon"
  [ benchp "single" L.colon ":"
  , benchp "failing" L.colon "."
  ]

benchDecimal :: Benchmark
benchDecimal = bgroup "decimal"
  [ benchp @Int "single" L.decimal "1"
  , benchp @Int "many" L.decimal "111111111"
  , benchp @Int "signed" (L.signed L.decimal) "-111111111"
  ]

benchFloat :: Benchmark
benchFloat = bgroup "float"
  [ benchp @Double "integer" L.float "1"
  , benchp @Double "real" L.float "2.87182364"
  , benchp @Double "signed" (L.signed L.float) "2.87182364"
  ]

benchHex :: Benchmark
benchHex = bgroup "hex"
  [ benchp @Int "single" L.hex "0x1"
  , benchp @Int "many" L.hex "0x123456789ABCDEF"
  ]

benchName :: Benchmark
benchName = bgroup "name"
  [ benchp "one letter" L.name "a"
  , benchp "100 letters" L.name "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
  , benchp "1 letter and 99 problems" L.name "a999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999"
  , benchp "love letters" L.name "love"
  ]

benchWhitespace :: Benchmark
benchWhitespace = bgroup "space"
  [ benchSpace
  , benchNewlines1
  ]

benchSpace :: Benchmark
benchSpace = bgroup "space"
  [ benchp "one space" L.space " "
  , benchp "some space" L.space "                                                                                                    "
  , benchp "failing" L.space "a"
  ]

benchNewlines1 :: Benchmark
benchNewlines1 = bgroup "newlines1"
  [ benchp "single newline" L.newlines1 "\n"
  , benchp "some newlines" L.newlines1 "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"
  , benchp "comment" L.newlines1 "# A comment\n"
  , benchp "newline, comment, space" L.newlines1 "\n# A comment\n   "
  , benchp "failing" L.newlines1 "a"
  ]

benchp :: NFData a => String -> Parser a -> Text -> Benchmark
benchp name p input = bench name (nf (parse p "") input)
